#!/usr/bin/env python
#
## Abstract Map Solver Class
# by mmckerns@caltech.edu

"""
This module contains the base class for mystic solvers that utilize
a parallel "map" function to enable parallel computing.  This module
describes the map solver interface.  As with the AbstractSolver, the
"Solve" method must be overwritte with the derived solver's optimization
algorithm. Additionally, for the AbstractMapSolver, a call to self.map
is required.  In many cases, a minimal function call interface for a
derived solver is provided along with the derived class.  See the
following for an example.

The default map API settings are provided within mystic, while
distributed and high-performance computing mappers and launchers
can be obtained within the "pathos" package, found here::
    - http://dev.danse.us/trac/pathos


Usage
=====

A typical call to a 'map' solver will roughly follow this example:

    >>> # the function to be minimized and the initial values
    >>> from mystic.models import rosen
    >>> lb = [0.0, 0.0, 0.0]
    >>> ub = [2.0, 2.0, 2.0]
    >>> 
    >>> # get monitors and termination condition objects
    >>> from mystic.tools import Sow
    >>> stepmon = Sow()
    >>> from mystic.termination import CandidateRelativeTolerance as CRT
    >>> 
    >>> # select the parallel launch configuration
    >>> from pyina.launchers import mpirun_launcher
    >>> from pyina.mappers import equalportion_mapper
    >>> from pyina.ez_map import ez_map2
    >>> NNODES = 4
    >>>> npts = 20
    >>>
    >>> # instantiate and configure the solver
    >>> from mystic.nested import ScattershotSolver
    >>> solver = ScattershotSolver(len(lb), npts)
    >>> solver.SetMapper(ez_map2, equalportion_mapper)
    >>> solver.SetLauncher(mpirun_launcher, NNODES)
    >>> solver.Solve(rosen, CRT(), StepMonitor=stepmon)
    >>> 
    >>> # obtain the solution
    >>> solution = solver.Solution()


Handler
=======

All solvers packaged with mystic include a signal handler that
provides the following options::
    sol: Print current best solution.
    cont: Continue calculation.
    call: Executes sigint_callback, if provided.
    exit: Exits with current best solution.

Handlers are enabled with the 'enable_signal_handler' method,
and are configured through the solver's 'Solve' method.  Handlers
trigger when a signal interrupt (usually, Ctrl-C) is given while
the solver is running.  ***NOTE: The handler currently is disabled
when the solver has been launched in parallel.*** 

"""
__all__ = ['AbstractMapSolver']


from mystic.tools import Null
from mystic.abstract_solver import AbstractSolver


class AbstractMapSolver(AbstractSolver):
    """
AbstractMapSolver base class for mystic optimizers that utilize parallel map.
    """

    def __init__(self, dim, **kwds):
        """
Takes one initial input:
    dim      -- dimensionality of the problem.

Additional inputs:
    npop     -- size of the trial solution population.     [default = 1]

Important class members:
    nDim, nPop     = dim, npop
    generations    - an iteration counter.
    bestEnergy     - current best energy.
    bestSolution   - current best parameter set.           [size = dim]
    popEnergy      - set of all trial energy solutions.    [size = npop]
    population     - set of all trial parameter solutions. [size = dim*npop]
    energy_history - history of bestEnergy status.         [equivalent to StepMonitor]
    signal_handler - catches the interrupt signal.         [***disabled***]
        """
        super(AbstractMapSolver, self).__init__(dim, **kwds)
       #AbstractSolver.__init__(self,dim,**kwds)
       #self.signal_handler   = None
       #self._handle_sigint   = False

        # import 'map' defaults
        from python_map import serial_launcher
        from python_map import python_map
        from python_map import carddealer_mapper
        from python_map import defaults

        # default settings for parallel and distributed computing
        self._launcher        = serial_launcher   # launcher
        self._map             = python_map        # map
        self._mapper          = carddealer_mapper # map_strategy
        self._nnodes          = int(defaults['nodes'])
        self._queue           = defaults['queue'] # scheduler_queue
        self._timelimit       = defaults['timelimit']
        self._servers         = ('*',) #<detect>  # hostname:port
        self._ncpus           = None   #<detect>  # local processors
       #self._servers         = ()     #<None>    # hostname:port
       #self._ncpus           = 0      #<None>    # local processors
        return

    def SelectServers(self, servers, ncpus=None): #XXX: needs some thought...
        """Select the compute server.

Description:

    Accepts a tuple of ('hostname:port',), listing each available
    computing server.

    If ncpus=None, then 'autodetect'; or if ncpus=0, then 'no local'.
    If servers=('*',), then 'autodetect'; or if servers=(), then 'no remote'.

Inputs:
    servers -- tuple of compute servers  [DEFAULT: autodetect]

Additional inputs:
    ncpus -- number of local processors  [DEFAULT: autodetect]
        """
        self._servers = servers
        self._ncpus = ncpus  #XXX: merge with nnodes, somehow ???
       #print "known servers: %s" % str(servers)
       #print "known # of local processors: %s" % str(ncpus)
        return

#   def SetFormat(self, format): #XXX: pickled versus source ???
#       self._format = format
#       return

    def SetMapper(self, map, strategy=None): #XXX: use strategy+format ?
        """Set the map function and the mapping strategy.

Description:

    Sets a mapping function to perform the map-reduce algorithm.
    Uses a mapping strategy to provide the algorithm for distributing
    the work list of optimization jobs across available resources.

Inputs:
    map -- the mapping function [DEFAULT: python_map]
    strategy -- map strategy (see pyina.mappers) [DEFAULT: carddealer_mapper]
        """
        self._map = map
        if strategy:
          self._mapper = strategy
        #FIXME: not a true mapping function... just a dummy interface to a str
        # a real mapper has map(func,*args) interface... this expects map().
        # should be...
        #   def xxx_map( f, input, *args, **kwds ):
        #     ...
        #     from yyy.xxx import map
        #     result = map(f, input, *args, **kwds)
        #     ...
        #     return result
        return

    def SetLauncher(self, launcher, nnodes=None): #XXX: use run+scheduler ?
        """Set launcher and (optionally) number of nodes.

Description:

    Uses a launcher to provide the solver with the syntax to
    configure and launch optimization jobs on the selected resource.

Inputs:
    launcher -- launcher function (see pyina.launchers)  [DEFAULT: serial_launcher]

Additional inputs:
    nnodes -- number of parallel compute nodes  [DEFAULT: 1]
        """
       #XXX: should be a Launcher class, not a function (see pathos.SSHLauncher)
        self._launcher = launcher
       #if launcher != python_map: #FIXME: CANNOT currently change to ez_map!!!
       #    exec("import pyina")   #FIXME: launcher should provide launcher.map
       #    exec("self._map = pyina.ez_map")
        self._nnodes = nnodes
        return

    def SelectQueue(self, queue, timelimit=None):  #XXX: SelectScheduler better?
        """Select queue and (optionally) timelimit.

Description:

    Takes string name of a scheduler queue to submit the optimization
    job. Additionally takes string time limit for scheduled job.

    Example: queue='normal', timelimit='00:02'

Inputs:
    queue -- queue name string (see pyina.launchers)  [DEFAULT: None]

Additional inputs:
    timelimit -- time string HH:MM:SS format  [DEFAULT: '00:05:00']
        """
        self._queue = queue
        self._timelimit = timelimit
        return


if __name__=='__main__':
    help(__name__)

# end of file



#!/usr/bin/env python

"""
One mogi source, similar to test_mogi, but uses
CostFactory objects

"""

from test_mogi import *

from mystic.forward_model import CostFactory
from mystic.filters import PickComponent

def de_solve(CF):
    solver = DifferentialEvolutionSolver(ND, NP)

    stepmon = Sow()
    minrange = [-1000., -1000., -100., -10.];
    maxrange = [1000., 1000., 100., 10.];
    solver.SetRandomInitialPoints(min = minrange, max = maxrange)
    solver.SetEvaluationLimits(maxiter=MAX_GENERATIONS)

    solver.Solve(CF, \
                 termination = ChangeOverGeneration(generations=100), \
                 CrossProbability=0.5, ScalingFactor=0.5, \
                 StepMonitor = stepmon)

    solution = solver.Solution()
  
    return solution, stepmon

if __name__ == '__main__':
    F = CostFactory()
    F.addModel(ForwardMogiFactory, 'mogi1', 4, outputFilter = PickComponent(2))
    myCostFunction = F.getCostFunction(evalpts = stations, observations = data_z)
    print F
    rp =  F.getRandomParams()
    print "new Cost Function : %s " % myCostFunction(rp)
    print "orig Cost Function: %s " % cost_function(rp)

    f1 = ForwardMogiFactory(rp)
    f2 = ForwardMogiFactory(rp)

    print 'start cf'
    for i in range(3000):
        xx = cost_function(rp)
    print 'end cf'

    print 'start cf2'
    for i in range(3000):
        xx = myCostFunction(rp)
    print 'end cf2'

    #de_solve(cost_function)
    de_solve(myCostFunction)

# end of file



#!/usr/bin/env python

"""
Example:
    - Solve 8th-order Chebyshev polynomial coefficients with DE.
    - Plot (x2) of convergence to Chebyshev polynomial.
    - Monitor (x2) Chi-Squared for Chebyshev polynomial.

Demonstrates:
    - standard models
    - expanded solver interface
    - built-in random initial guess
    - customized monitors and termination conditions
    - customized DE mutation strategies
"""

# Differential Evolution solver
from mystic.differential_evolution import DifferentialEvolutionSolver2

# Chebyshev polynomial and cost function
from mystic.models.poly import chebyshev8, chebyshev8cost
from mystic.models.poly import chebyshev8coeffs

# tools
from mystic.termination import VTR
from mystic.strategy import Best1Exp
from mystic import getch, random_seed, VerboseSow, Sow
from mystic.math import poly1d
import pylab
pylab.ion()

# draw the plot
def plot_frame(label=None):
    pylab.close()
    pylab.title("8th-order Chebyshev coefficient convergence")
    pylab.xlabel("Differential Evolution %s" % label)
    pylab.ylabel("Chi-Squared")
    return
 
# plot the polynomial trajectories
def plot_params(monitor):
    x = range(len(monitor.y))
    pylab.plot(x,monitor.y,'b-')
    pylab.axis([1,0.5*x[-1],0,monitor.y[1]],'k-')
    return


if __name__ == '__main__':

    print "Differential Evolution"
    print "======================"

    # set range for random initial guess
    ndim = 9
    x0 = [(-100,100)]*ndim
    random_seed(123)

    # configure monitors
    stepmon = VerboseSow(50)
    evalmon = Sow()

    # use DE to solve 8th-order Chebyshev coefficients
    npop = 10*ndim
    solver = DifferentialEvolutionSolver2(ndim,npop)
    solver.SetRandomInitialPoints(min=[-100]*ndim, max=[100]*ndim)
    solver.SetEvaluationLimits(maxiter=999)
    solver.enable_signal_handler()
    solver.Solve(chebyshev8cost,termination=VTR(0.01),strategy=Best1Exp, \
                 CrossProbability=1.0, ScalingFactor=0.9, \
                 StepMonitor=stepmon, EvaluationMonitor=evalmon)
    solution = solver.Solution()

    # print solved coefficients and Chi-Squared
    iterations = len(stepmon.x)
    cost = solver.bestEnergy
    print "\nGeneration %d has best Chi-Squared: %f" % (iterations, cost)
    print "Solved Coefficients:\n %s\n" % poly1d(solver.bestSolution)

    # plot convergence of coefficients per iteration
    plot_frame('iterations')
    plot_params(stepmon) 
    getch()

    # plot convergence of coefficients per function call
    plot_frame('function calls')
    plot_params(evalmon) 
    getch()

# end of file



#!/usr/bin/env python

"""
Testing the polynomial fitting problem of [1] using scipy's Nelder-Mead algorithm.

Reference:

[1] Storn, R. and Price, K. Differential Evolution - A Simple and Efficient
Heuristic for Global Optimization over Continuous Spaces. Journal of Global
Optimization 11: 341-359, 1997.
"""

from test_ffit import Chebyshev8, ChebyshevCost, plot_solution, print_solution

if __name__ == '__main__':
    from mystic.scipy_optimize import fmin
   #from scipy.optimize import fmin
    import random
    random.seed(123)
    x = [random.uniform(-100,100) + Chebyshev8[i] for i in range(9)]
    solution = fmin(ChebyshevCost, x)
    print_solution(solution)
    plot_solution(solution)

# end of file



#!/usr/bin/env python

"""
Example:
    - Solve 8th-order Chebyshev polynomial coefficients with Powell's method.
    - Plot of fitting to Chebyshev polynomial.

Demonstrates:
    - standard models
    - minimal solver interface
"""

# Powell's Directonal solver
from mystic.scipy_optimize import fmin_powell

# Chebyshev polynomial and cost function
from mystic.models.poly import chebyshev8, chebyshev8cost
from mystic.models.poly import chebyshev8coeffs

# tools
from mystic.math import poly1d
from mystic import getch
import pylab
pylab.ion()

# draw the plot
def plot_exact():
    pylab.title("fitting 8th-order Chebyshev polynomial coefficients")
    pylab.xlabel("x")
    pylab.ylabel("f(x)")
    import numpy
    x = numpy.arange(-1.2, 1.2001, 0.01)
    exact = chebyshev8(x)
    pylab.plot(x,exact,'b-')
    pylab.legend(["Exact"])
    pylab.axis([-1.4,1.4,-2,8],'k-')
    return
 
# plot the polynomial
def plot_solution(params):
    import numpy
    x = numpy.arange(-1.2, 1.2001, 0.01)
    f = poly1d(params)
    y = f(x)
    pylab.plot(x,y,'y-')
    pylab.legend(["Exact","Fitted"])
    pylab.axis([-1.4,1.4,-2,8],'k-')
    return


if __name__ == '__main__':

    print "Powell's Method"
    print "==============="

    # initial guess
    import random
    random.seed(123)
    ndim = 9
    x0 = [random.uniform(-100,100) for i in range(ndim)]

    # draw frame and exact coefficients
    plot_exact()

    # use Powell's method to solve 8th-order Chebyshev coefficients
    solution = fmin_powell(chebyshev8cost,x0)

    # use pretty print for polynomials
    print poly1d(solution)

    # compare solution with actual 8th-order Chebyshev coefficients
    print "\nActual Coefficients:\n %s\n" % poly1d(chebyshev8coeffs)

    # plot solution versus exact coefficients
    plot_solution(solution) 
    getch()

# end of file



#!/usr/bin/env python

"""
Corana's function

References::
    [1] Storn, R. and Price, K. Differential Evolution - A Simple and Efficient
    Heuristic for Global Optimization over Continuous Spaces. Journal of Global
    Optimization 11: 341-359, 1997.

    [2] Storn, R. and Price, K.
    (Same title as above, but as a technical report.)
    http://www.icsi.berkeley.edu/~storn/deshort1.ps
"""
from abstract_model import AbstractFunction

from numpy import asarray
from math import pow
from numpy import sign, floor

class Corana(AbstractFunction):
    """Corana's function:
a multi-minima function, Equation (22) of [2]"""

    def __init__(self):
        AbstractFunction.__init__(self)
        return

    def function(self,coeffs):
        """evaluates the Corana function for a list of coeffs

minimum is f(x)=0.0 at xi=0.0"""
        d = [1., 1000., 10., 100.]
       #x = asarray(coeffs) #XXX: converting to numpy.array slows by 10x
        x = coeffs
        r = 0
        for j in range(4):
            zj =  floor( abs(x[j]/0.2) + 0.49999 ) * sign(x[j]) * 0.2
            if abs(x[j]-zj) < 0.05:
                r += 0.15 * pow(zj - 0.05*sign(zj), 2) * d[j]
            else:
                r += d[j] * x[j] * x[j]
        return r

#   def forward(self,pts):
#       """n-dimensional Corana; returns f(xi) for each xi in pts"""
#       return AbstractFunction.forward(self,pts)

    pass


# prepared instances
corana = Corana()

def corana1d(x):
    """Corana in 1D; coeffs = (x,0,0,0)"""
    return corana([x[0], 0, 0, 0])

def corana2d(x):
    """Corana in 2D; coeffs = (x,0,y,0)"""
    return corana([x[0], 0, x[1], 0])

def corana3d(x):
    """Corana in 3D; coeffs = (x,0,y,z)"""
    return corana([x[0], 0, x[1], x[2]])

# End of file



#!/usr/bin/env python

"""
2d array representation of a circle

References::
    None
"""
#FIXME: cost function seems to apply penalty when r=R0... it should not
from abstract_model import AbstractModel

from numpy import array, pi, arange
from numpy import random, sin, cos
from math import floor, sqrt

random.seed(123)

class Circle(AbstractModel):
    """Computes 2D array representation of a circle
where the circle minimally bounds the 2D data points

data points with [minimal, sparse, or dense] packing=[~0.2, ~1.0, or ~5.0]
setting packing = None will constrain all points to the circle's radius"""
    def __init__(self,packing=None,name='circle',sigma=1.0):
        AbstractModel.__init__(self,name,sigma)
        if packing == None: packing = 0.0
        self.__packing__ = packing
        return

    def __call__(self,x,y,r,*args,**kwds):
        return self.forward((x,y,r),*args,**kwds)

    def forward(self,coeffs,npts=None):
        """generate a 2D array of points contained within a circle
built from a list of coefficients
(x,y,r) = coeffs

default npts = packing * floor( pi*radius^2 )"""
        if not npts:
            # generate # of points based on packing and given radius
            npts = self.__packing__ * floor(pi*(coeffs[-1])**2)
        return gendata(coeffs,npts)

    def ForwardFactory(self,coeffs):
        """generates a circle instance from a list of coefficients
(x,y,r) = coeffs"""
        x,y,r = coeffs
        def forward_circle(npts=None):
            """generate a 2D array representation of a circle
with (x,y,r) = (%s,%s,%s)""" % (x,y,r)
            return self.forward((x,y,r),npts)
        return forward_circle

    def CostFactory(self,target,npts=None):
        """generates a cost function instance from list of coefficients & number of evaluation points
(x,y,r) = target coeffs"""
        datapts = self.forward(target,npts)
        def cost(params):
            """cost function for minimum enclosing circle for a 2D set of points"""
            x,y,r = params
            if r<0:
                return -999. * r
            penalty = 0
            for xx,yy in datapts:
               # compute distance to origin
               d = sqrt((xx-x)*(xx-x) + (yy-y)*(yy-y))
               if d > r:
                   # each violation adds 1 to the cost plus amount of violation
                   penalty += 1+d-r
            return self.__sigma__ * (r+penalty)
        self.__cost__ = cost
        return self.__cost__

    def CostFactory2(self,datapts):
        """generates a cost function instance from a 2D array of datapoints"""
        def cost(params):
            """cost function for minimum enclosing circle for a 2D set of points"""
            x,y,r = params
            if r<0:
                return -999. * r
            penalty = 0
            for xx,yy in datapts:
               # compute distance to origin
               d = sqrt((xx-x)*(xx-x) + (yy-y)*(yy-y))
               if d > r:
                   # each violation adds 1 to the cost plus amount of violation
                   penalty += 1+d-r
            return self.__sigma__ * (r+penalty)
        self.__cost__ = cost
        return self.__cost__

    pass

# prepared instances
circle = Circle()
dense_circle = Circle(packing=5.0)
sparse_circle = Circle(packing=1.0)
minimal_circle = Circle(packing=0.2)


# helper functions
def gencircle(coeffs,interval=0.02):
    """generate a 2D array representation of a circle of given coeffs
coeffs = (x,y,r)"""
    x,y,r = coeffs
    theta = arange(0, 2*pi, interval)
    xy = array(zip(r*cos(theta)+x, r*sin(theta)+y))
    return xy

def gendata(coeffs,npts=20):
    """Generate a 2D dataset of npts enclosed in circle of given coeffs,
where coeffs = (x,y,r).

NOTE: if npts == None, constrain all points to circle of given radius"""
    if not npts:
        return gencircle(coeffs)
    def points_circle(N):
        # generate N random points in a unit circle
        n = 0
        while n < N:
            x = random.random()*2.-1.
            y = random.random()*2.-1.
            if x*x + y*y <= 1:
                n = n+1
                yield [x,y]
    x0,y0,R0 = coeffs
    xy = array(list(points_circle(npts)))*R0
    xy[:,0] += x0
    xy[:,1] += y0
    return xy


# End of file



#!/usr/bin/env python

"""
Similar to test_mogi.py

but trying to use scipy's levenberg marquardt.

"""

from test_mogi import *

# Here is the new cost function definition
def cost_function_levenbergmarquardt(params):
    model = ForwardMogiFactory(params)
    zdisp = filter_for_zdisp(model(stations))
    return zdisp - data_z

import pylab
from scipy.optimize import fmin_cg

if __name__ == '__main__':

    sol = fmin_cg(cost_function, [1000., -500., -10., 0.1], full_output=1)
    print "scipy solution: ", sol[0]
    plot_sol(sol[0],'r-')
    pylab.show()

# end of file



#!/usr/bin/env python

"""
Example:
    - Minimize Rosenbrock's Function with Nelder-Mead.
    - Plot of parameter convergence to function minimum.

Demonstrates:
    - standard models
    - minimal solver interface
    - parameter trajectories using retall
"""

# Nelder-Mead solver
from mystic.scipy_optimize import fmin

# Rosenbrock function
from mystic.models import rosen

# tools
import pylab


if __name__ == '__main__':

    # initial guess
    x0 = [0.8,1.2,0.7]

    # use Nelder-Mead to minimize the Rosenbrock function
    solution = fmin(rosen,x0,disp=0,retall=1)
    allvecs = solution[-1]

    # plot the parameter trajectories
    pylab.plot([i[0] for i in allvecs])
    pylab.plot([i[1] for i in allvecs])
    pylab.plot([i[2] for i in allvecs])

    # draw the plot
    pylab.title("Rosenbrock parameter convergence")
    pylab.xlabel("Nelder-Mead solver iterations")
    pylab.ylabel("parameter value")
    pylab.legend(["x", "y", "z"])
    pylab.show()
 
# end of file



#!/usr/bin/env python

"""
1d model representation for polynomials

References::
    [1] Storn, R. and Price, K. Differential Evolution - A Simple and Efficient
    Heuristic for Global Optimization over Continuous Spaces. Journal of Global
    Optimization 11: 341-359, 1997.

    [2] Storn, R. and Price, K.
    (Same title as above, but as a technical report.)
    http://www.icsi.berkeley.edu/~storn/deshort1.ps
"""
from abstract_model import AbstractModel

from numpy import sum as numpysum
from numpy import asarray
from mystic.math import polyeval, poly1d


class Polynomial(AbstractModel):
    """1-D Polynomial models and functions"""

    def __init__(self,name='poly',metric=lambda x: numpysum(x*x),sigma=1.0):
        AbstractModel.__init__(self,name,metric,sigma)
        return

    def evaluate(self,coeffs,x):
        """takes list of coefficients & evaluation points, returns f(x)
thus, [a3, a2, a1, a0] yields  a3 x^3 + a2 x^2 + a1 x^1 + a0"""
        return polyeval(coeffs,x)

    def ForwardFactory(self,coeffs):
        """generates a 1-D polynomial instance from a list of coefficients
using numpy.poly1d(coeffs)"""
        self.__forward__ = poly1d(coeffs)
        return self.__forward__

    pass


# coefficients for specific Chebyshev polynomials
chebyshev8coeffs = [128., 0., -256., 0., 160., 0., -32., 0., 1.]
chebyshev16coeffs = [32768., 0., -131072., 0., 212992., 0., -180224., 0., 84480., 0., -21504., 0., 2688., 0., -128., 0., 1]

class Chebyshev(Polynomial):
    """Chebyshev polynomial models and functions,
including specific methods for T8(z) & T16(z), Equation (27-33) of [2]

NOTE: default is T8(z)"""

    def __init__(self,order=8,name='poly',metric=lambda x: numpysum(x*x),sigma=1.0):
        Polynomial.__init__(self,name,metric,sigma)
        if order == 8:  self.coeffs = chebyshev8coeffs
        elif order == 16:  self.coeffs = chebyshev16coeffs
        else: raise NotImplementedError, "provide self.coeffs 'by hand'"
        return

    def __call__(self,*args,**kwds):
        return self.forward(*args,**kwds)

    def forward(self,x):
        """forward Chebyshev function""" #of order %s""" % (len(self.coeffs)-1)
        fwd = Polynomial.ForwardFactory(self,self.coeffs)
        return fwd(x)

    def ForwardFactory(self,coeffs):
        """generates a 1-D polynomial instance from a list of coefficients"""
        raise NotImplementedError, "use Polynomial.ForwardFactory(coeffs)"

    def CostFactory(self,target,pts):
        """generates a cost function instance from list of coefficients & evaluation points"""
        raise NotImplementedError, "use Polynomial.CostFactory(targets,pts)"

    def CostFactory2(self,pts,datapts,nparams):
        """generates a cost function instance from datapoints & evaluation points"""
        raise NotImplementedError, "use Polynomial.CostFactory2(pts,datapts,nparams)"

    def cost(self,trial,M=61):
        """The costfunction for order-n Chebyshev fitting.
M evaluation points between [-1, 1], and two end points"""# % (len(self.coeffs)-1)
        #XXX: throw error when len(trial) != len(self.coeffs) ?
        myCost = chebyshevcostfactory(self.coeffs)
        return myCost(trial,M)

    pass
 
# faster implementation
def chebyshevcostfactory(target):
    def chebyshevcost(trial,M=61):
        """The costfunction for order-n Chebyshev fitting.
M evaluation points between [-1, 1], and two end points"""

        result=0.0
        x=-1.0
        dx = 2.0 / (M-1)
        for i in range(M):
            px = polyeval(trial, x)
            if px<-1 or px>1:
                result += (1 - px) * (1 - px)
            x += dx

        px = polyeval(trial, 1.2) - polyeval(target, 1.2)
        if px<0: result += px*px

        px = polyeval(trial, -1.2) - polyeval(target, -1.2)
        if px<0: result += px*px

        return result
    return chebyshevcost

# prepared instances
poly = Polynomial()
chebyshev8 = Chebyshev(8)
chebyshev16 = Chebyshev(16)

chebyshev8cost = chebyshevcostfactory(chebyshev8coeffs)
chebyshev16cost = chebyshevcostfactory(chebyshev16coeffs)


# End of file



#!/usr/bin/env python
#

"""
Solvers
=======

This module contains a collection of optimization that use map-reduce
to distribute several optimizer instances over parameter space. Each
solver accepts a imported solver object as the "nested" solver, which
becomes the target of the map function.

The set of solvers built on mystic's AbstractNestdSolver are::
   BatchGridSolver -- start from center of N grid points
   ScattershotSolver -- start from N random points in parameter space


Usage
=====

See `mystic.examples.scattershot_example06` for an example of using
ScattershotSolver. See `mystic.examples.batchgrid_example06`
or an example of using BatchGridSolver.

All solvers included in this module provide the standard signal handling.
For more information, see `mystic.mystic.abstract_solver`.
"""
__all__ = ['BatchGridSolver','ScattershotSolver']

from mystic.tools import Null, wrap_function

from abstract_nested_solver import AbstractNestedSolver


class BatchGridSolver(AbstractNestedSolver):
    """
parallel mapped optimization starting from the center of N grid points
    """
    def __init__(self, dim, nbins):
        """
Takes two initial inputs: 
    dim   -- dimensionality of the problem
    nbins -- tuple of number of bins in each dimension

All important class members are inherited from AbstractNestedSolver.
        """
        super(BatchGridSolver, self).__init__(dim, nbins=nbins)

#   def SetGridBins(self, nbins):
#   def ConfigureNestedSolver(self, **kwds):
#       """Override me for more refined behavior.
#       """
#       return

    def Solve(self, cost, termination, sigint_callback=None,
              EvaluationMonitor=Null, StepMonitor=Null, ExtraArgs=(), **kwds):
        """Minimize a function using batch grid optimization.

Description:

    Uses parallel mapping of solvers on a regular grid to find the
    minimum of a function of one or more variables.

Inputs:

    cost -- the Python function or method to be minimized.
    termination -- callable object providing termination conditions.

Additional Inputs:

    sigint_callback -- callback function for signal handler.
    EvaluationMonitor -- a callable object that will be passed x, fval
        whenever the cost function is evaluated.
    StepMonitor -- a callable object that will be passed x, fval
        after the end of a solver iteration.
    ExtraArgs -- extra arguments for cost.

Further Inputs:

    callback -- an optional user-supplied function to call after each
        iteration.  It is called as callback(xk), where xk is the
        current parameter vector.                           [default = None]
    disp -- non-zero to print convergence messages.         [default = 0]
        """
        #allow for inputs that don't conform to AbstractSolver interface
#       callback=None        #user-supplied function, called after each step
        disp=0               #non-zero to print convergence messages
#       if kwds.has_key('callback'): callback = kwds['callback']
        if kwds.has_key('disp'): disp = kwds['disp']
        #-------------------------------------------------------------

        import signal
       #self._EARLYEXIT = False

       #FIXME: EvaluationMonitor fails for MPI, throws error for 'pp'
        from python_map import python_map
        if self._map != python_map:
            fcalls = [0] #FIXME: temporary patch for removing the following line
        else:
            fcalls, cost = wrap_function(cost, ExtraArgs, EvaluationMonitor)

        #generate signal_handler
        self._generateHandler(sigint_callback) 
        if self._handle_sigint: signal.signal(signal.SIGINT,self.signal_handler)

        #-------------------------------------------------------------

        nbins = self._nbins
        if len(self._strictMax): upper = list(self._strictMax)
        else:
            upper = list(self._defaultMax)
        if len(self._strictMin): lower = list(self._strictMin)
        else:
            lower = list(self._defaultMin)

        # generate arrays of points defining a grid in parameter space
        grid_dimensions = self.nDim
        bins = []
        for i in range(grid_dimensions):
            step = abs(upper[i] - lower[i])/nbins[i]
            bins.append( [lower[i] + (j+0.5)*step for j in range(nbins[i])] )

        # build a grid of starting points
        from mystic.math import gridpts
        initial_values = gridpts(bins)

        # run optimizer for each grid point
        cf = [cost for i in range(len(initial_values))]
        tm = [termination for i in range(len(initial_values))]

        # generate the local_optimize function
        local_opt = """\n
def local_optimize(cost, termination, x0):
    from %s import %s as LocalSolver
    from mystic import Sow

    stepmon = Sow()
    evalmon = Sow()

    ndim = len(x0)

    solver = LocalSolver(ndim)
    solver.SetInitialPoints(x0)
""" % (self._solver.__module__, self._solver.__name__)
        if self._useStrictRange:
            local_opt += """\n
    solver.SetStrictRanges(min=%s, max=%s)
""" % (str(lower), str(upper))
        local_opt += """\n
    solver.SetEvaluationLimits(%s, %s)
    solver.Solve(cost, termination, StepMonitor=stepmon, EvaluationMonitor=evalmon, disp=%s)
    solved_params = solver.Solution()
    solved_energy = solver.bestEnergy
    return solved_params, solved_energy, stepmon, evalmon
""" % (str(self._maxiter), str(self._maxfun), str(disp))
        exec local_opt

        # map:: params, energy, smon, emon = local_optimize(cost,term,x0)
        mapconfig = dict(nnodes=self._nnodes, launcher=self._launcher, \
                         mapper=self._mapper, queue=self._queue, \
                         timelimit=self._timelimit, \
                         ncpus=self._ncpus, servers=self._servers)
        results = self._map(local_optimize, cf, tm, initial_values, **mapconfig)

        # get the results with the lowest energy
        best = list(results[0][0]), results[0][1]
        bestpath = results[0][2]
        besteval = results[0][3]
        func_evals = len(besteval.y)
        for result in results[1:]:
          func_evals += len((result[3]).y)  # add function evaluations
          if result[1] < best[1]: # compare energy
            best = list(result[0]), result[1]
            bestpath = result[2]
            besteval = result[3]

        # return results to internals
        self.bestSolution = best[0]
        self.bestEnergy = best[1]
        self.generations = len(bestpath.y)
        fcalls = [ len(besteval.y) ]

        # write 'bests' to monitors  #XXX: non-best monitors may be useful too
        for i in range(len(bestpath.y)):
            StepMonitor(bestpath.x[i], bestpath.y[i])
            #XXX: could apply callback here, or in exec'd code
        for i in range(len(besteval.y)):
            EvaluationMonitor(besteval.x[i], besteval.y[i])

        #-------------------------------------------------------------

        signal.signal(signal.SIGINT,signal.default_int_handler)

        # code below here pushes output to scipy.optimize.fmin interface
        fval = self.bestEnergy
        warnflag = 0

        # little hack to not set off Warnings when maxiter/maxfun not set
        if self._maxiter is None: self._maxiter = self.nDim * 1e8
        if self._maxfun is None: self._maxfun = self.nDim * 1e8

        if fcalls[0] >= self._maxfun:
            warnflag = 1
            if disp:
                print "Warning: Maximum number of function evaluations has "\
                      "been exceeded."
        elif self.generations >= self._maxiter:
            warnflag = 2
            if disp:
                print "Warning: Maximum number of iterations has been exceeded"
        else:
            if disp:
                print "Optimization terminated successfully."
                print "         Current function value: %f" % fval
                print "         Iterations: %d" % self.generations
                print "         Function evaluations: %d" % fcalls[0]

        return 

class ScattershotSolver(AbstractNestedSolver):
    """
parallel mapped optimization starting from the N random points
    """
    def __init__(self, dim, npts):
        """
Takes two initial inputs: 
    dim   -- dimensionality of the problem
    npts  -- number of parallel solver instances

All important class members are inherited from AbstractNestedSolver.
        """
        super(ScattershotSolver, self).__init__(dim, npts=npts)

    def Solve(self, cost, termination, sigint_callback=None,
              EvaluationMonitor=Null, StepMonitor=Null, ExtraArgs=(), **kwds):
        """Minimize a function using scattershot optimization.

Description:

    Uses parallel mapping of solvers on randomly selected points
    to find the minimum of a function of one or more variables.

Inputs:

    cost -- the Python function or method to be minimized.
    termination -- callable object providing termination conditions.

Additional Inputs:

    sigint_callback -- callback function for signal handler.
    EvaluationMonitor -- a callable object that will be passed x, fval
        whenever the cost function is evaluated.
    StepMonitor -- a callable object that will be passed x, fval
        after the end of a solver iteration.
    ExtraArgs -- extra arguments for cost.

Further Inputs:

    callback -- an optional user-supplied function to call after each
        iteration.  It is called as callback(xk), where xk is the
        current parameter vector.                           [default = None]
    disp -- non-zero to print convergence messages.         [default = 0]
        """
        #allow for inputs that don't conform to AbstractSolver interface
#       callback=None        #user-supplied function, called after each step
        disp=0               #non-zero to print convergence messages
#       if kwds.has_key('callback'): callback = kwds['callback']
        if kwds.has_key('disp'): disp = kwds['disp']
        #-------------------------------------------------------------

        import signal
       #self._EARLYEXIT = False

       #FIXME: EvaluationMonitor fails for MPI, throws error for 'pp'
        from python_map import python_map
        if self._map != python_map:
            fcalls = [0] #FIXME: temporary patch for removing the following line
        else:
            fcalls, cost = wrap_function(cost, ExtraArgs, EvaluationMonitor)

        #generate signal_handler
        self._generateHandler(sigint_callback) 
        if self._handle_sigint: signal.signal(signal.SIGINT,self.signal_handler)

        #-------------------------------------------------------------

        npts = self._npts
        if len(self._strictMax): upper = list(self._strictMax)
        else:
            upper = list(self._defaultMax)
        if len(self._strictMin): lower = list(self._strictMin)
        else:
            lower = list(self._defaultMin)

        # generate a set of starting points
        from mystic.math import samplepts
        initial_values = samplepts(lower,upper,npts)

        # run optimizer for each grid point
        cf = [cost for i in range(len(initial_values))]
        tm = [termination for i in range(len(initial_values))]

        # generate the local_optimize function
        local_opt = """\n
def local_optimize(cost, termination, x0):
    from %s import %s as LocalSolver
    from mystic import Sow

    stepmon = Sow()
    evalmon = Sow()

    ndim = len(x0)

    solver = LocalSolver(ndim)
    solver.SetInitialPoints(x0)
""" % (self._solver.__module__, self._solver.__name__)
        if self._useStrictRange:
            local_opt += """\n
    solver.SetStrictRanges(min=%s, max=%s)
""" % (str(lower), str(upper))
        local_opt += """\n
    solver.SetEvaluationLimits(%s, %s)
    solver.Solve(cost, termination, StepMonitor=stepmon, EvaluationMonitor=evalmon, disp=%s)
    solved_params = solver.Solution()
    solved_energy = solver.bestEnergy
    return solved_params, solved_energy, stepmon, evalmon
""" % (str(self._maxiter), str(self._maxfun), str(disp))
        exec local_opt

        # map:: params, energy, smon, emon = local_optimize(cost,term,x0)
        mapconfig = dict(nnodes=self._nnodes, launcher=self._launcher, \
                         mapper=self._mapper, queue=self._queue, \
                         timelimit=self._timelimit, \
                         ncpus=self._ncpus, servers=self._servers)
        results = self._map(local_optimize, cf, tm, initial_values, **mapconfig)

        # get the results with the lowest energy
        best = list(results[0][0]), results[0][1]
        bestpath = results[0][2]
        besteval = results[0][3]
        func_evals = len(besteval.y)
        for result in results[1:]:
          func_evals += len((result[3]).y)  # add function evaluations
          if result[1] < best[1]: # compare energy
            best = list(result[0]), result[1]
            bestpath = result[2]
            besteval = result[3]

        # return results to internals
        self.bestSolution = best[0]
        self.bestEnergy = best[1]
        self.generations = len(bestpath.y)
        fcalls = [ len(besteval.y) ]

        # write 'bests' to monitors  #XXX: non-best monitors may be useful too
        for i in range(len(bestpath.y)):
            StepMonitor(bestpath.x[i], bestpath.y[i])
            #XXX: could apply callback here, or in exec'd code
        for i in range(len(besteval.y)):
            EvaluationMonitor(besteval.x[i], besteval.y[i])

        #-------------------------------------------------------------

        signal.signal(signal.SIGINT,signal.default_int_handler)

        # code below here pushes output to scipy.optimize.fmin interface
        fval = self.bestEnergy
        warnflag = 0

        # little hack to not set off Warnings when maxiter/maxfun not set
        if self._maxiter is None: self._maxiter = self.nDim * 1e8
        if self._maxfun is None: self._maxfun = self.nDim * 1e8

        if fcalls[0] >= self._maxfun:
            warnflag = 1
            if disp:
                print "Warning: Maximum number of function evaluations has "\
                      "been exceeded."
        elif self.generations >= self._maxiter:
            warnflag = 2
            if disp:
                print "Warning: Maximum number of iterations has been exceeded"
        else:
            if disp:
                print "Optimization terminated successfully."
                print "         Current function value: %f" % fval
                print "         Iterations: %d" % self.generations
                print "         Function evaluations: %d" % fcalls[0]

        return 


if __name__=='__main__':
    help(__name__)

# end of file



#!/usr/bin/env python
"""
math: mathematical functions and tools for use in mystic


Functions
=========

Mystic provides a set of mathematical functions that support various
advanced optimization features such as uncertainty analysis and
parameter sensitivity.  (These functions are to be provided in an
upcoming release.)


Tools
=====

Mystic also provides a set of mathematical tools that support advanced
features such as parameter space partitioning and monte carlo estimation.
These mathematical tools are provided::
    polyeval     -- fast evaluation of an n-dimensional polynomial
    poly1d       -- generate a 1d polynomial instance
    gridpts      -- generate a set of regularly spaced points
    samplepts    -- generate a set of randomly sampled points 
    approx_equal -- test if equal within some absolute or relative tolerance


"""
# functions and tools
from poly import polyeval, poly1d
from grid import gridpts, samplepts
from approx import approx_equal

# end of file



#!/usr/bin/env python
""" NOTE:
rosen rewritten as a pickleable function
"""

def rosen(coeffs):
    """evaluates n-dimensional Rosenbrock function for a list of coeffs

minimum is f(x)=0.0 at xi=1.0"""
    from numpy import sum as numpysum
    from numpy import asarray

    #ensure that there are 2 coefficients
    x = [1]*2
    x[:len(coeffs)]=coeffs
    x = asarray(x) #XXX: must be a numpy.array
    return numpysum(100.0*(x[1:]-x[:-1]**2.0)**2.0 + (1-x[:-1])**2.0)


if __name__=='__main__':
    target = [1., 1., 1.]
    print target
    print ""
    print rosen(target)



#!/usr/bin/env python
#
# Michael McKerns
# mmckerns@caltech.edu

# check if easy_install is available
try:
#   import __force_distutils__ #XXX: uncomment to force use of distutills
    from setuptools import setup
    has_setuptools = True
except ImportError:
    from distutils.core import setup
    has_setuptools = False

# build the 'setup' call
setup_code = """
setup(name='mystic',
      version='0.2a1',
      description='a simple interactive inversion analysis framework',
      author = 'Mike McKerns',
      maintainer = 'Mike McKerns',
      maintainer_email = 'mmckerns@caltech.edu',
      license = 'BSD',
      platforms = ['any'],
      url = 'http://www.cacr.caltech.edu/~mmckerns',
      classifiers = ('Intended Audience :: Developers',
                     'Programming Language :: Python',
                     'Topic :: Physics Programming'),

      packages = ['mystic','mystic.models','mystic.math'],
      package_dir = {'mystic':'mystic','mystic.models':'models',
                     'mystic.math':'_math'},
"""

# add dependencies
numpy_version = '>=1.0'
matplotlib_version = '>=0.91'
if has_setuptools:
    setup_code += """
      install_requires = ('numpy%s'),
""" % numpy_version

# close 'setup' call
setup_code += """    
      zip_safe=True,
      scripts=[])
"""

# exec the 'setup' code
exec setup_code

# if dependencies are missing, print a warning
try:
    import numpy
    #import matplotlib #XXX: has issues being zip_safe
except ImportError:
    print "\n***********************************************************"
    print "WARNING: One of the following dependencies is unresolved:"
    print "    numpy %s" % numpy_version
    print "    matplotlib %s (optional)" % matplotlib_version
    print "***********************************************************\n"


if __name__=='__main__':
    pass

# end of file



#!/usr/bin/env python
""" NOTE:
due to pickling issues, cost function is provided w/o using a factory method.
(same as chebyshev8.py, except uses global target,polyeval,poly1d)
"""

from mystic.models.poly import chebyshev8coeffs as target
from mystic.math import polyeval, poly1d

def chebyshev8cost(trial,M=61):
    """The costfunction for order-n Chebyshev fitting.
M evaluation points between [-1, 1], and two end points"""

    result=0.0
    x=-1.0
    dx = 2.0 / (M-1)
    for i in range(M):
        px = polyeval(trial, x)
        if px<-1 or px>1:
            result += (1 - px) * (1 - px)
        x += dx

    px = polyeval(trial, 1.2) - polyeval(target, 1.2)
    if px<0: result += px*px

    px = polyeval(trial, -1.2) - polyeval(target, -1.2)
    if px<0: result += px*px

    return result

if __name__=='__main__':
    print poly1d(target)
    print ""
    print chebyshev8cost(target)



#!/usr/bin/env python

"""
Example:
    - Solve 8th-order Chebyshev polynomial coefficients with DE.
    - Callable plot of fitting to Chebyshev polynomial.
    - Monitor Chi-Squared for Chebyshev polynomial.

Demonstrates:
    - standard models
    - expanded solver interface
    - built-in random initial guess
    - customized monitors and termination conditions
    - customized DE mutation strategies
"""

# Differential Evolution solver
from mystic.differential_evolution import DifferentialEvolutionSolver2

# Chebyshev polynomial and cost function
from mystic.models.poly import chebyshev8, chebyshev8cost
from mystic.models.poly import chebyshev8coeffs

# tools
from mystic.termination import VTR
from mystic.strategy import Best1Exp
from mystic import getch, random_seed, VerboseSow
from mystic.math import poly1d
import pylab
pylab.ion()

# draw the plot
def plot_exact():
    pylab.title("fitting 8th-order Chebyshev polynomial coefficients")
    pylab.xlabel("x")
    pylab.ylabel("f(x)")
    import numpy
    x = numpy.arange(-1.2, 1.2001, 0.01)
    exact = chebyshev8(x)
    pylab.plot(x,exact,'b-')
    pylab.legend(["Exact"])
    pylab.axis([-1.4,1.4,-2,8],'k-')
    return
 
# plot the polynomial
def plot_solution(params):
    import numpy
    x = numpy.arange(-1.2, 1.2001, 0.01)
    f = poly1d(params)
    y = f(x)
    pylab.plot(x,y,'y-')
    pylab.legend(["Exact","Fitted"])
    pylab.axis([-1.4,1.4,-2,8],'k-')
    return


if __name__ == '__main__':

    print "Differential Evolution"
    print "======================"

    # set range for random initial guess
    ndim = 9
    x0 = [(-100,100)]*ndim
    random_seed(123)

    # draw frame and exact coefficients
    plot_exact()

    # configure monitor
    stepmon = VerboseSow(50)

    # use DE to solve 8th-order Chebyshev coefficients
    npop = 10*ndim
    solver = DifferentialEvolutionSolver2(ndim,npop)
    solver.SetRandomInitialPoints(min=[-100]*ndim, max=[100]*ndim)
    solver.enable_signal_handler()
    solver.Solve(chebyshev8cost,termination=VTR(0.01),strategy=Best1Exp, \
                 CrossProbability=1.0, ScalingFactor=0.9, \
                 StepMonitor=stepmon, sigint_callback=plot_solution)
    solution = solver.Solution()

    # use monitor to retrieve results information
    iterations = len(stepmon.x)
    cost = stepmon.y[-1]
    print "Generation %d has best Chi-Squared: %f" % (iterations, cost)

    # use pretty print for polynomials
    print poly1d(solution)

    # compare solution with actual 8th-order Chebyshev coefficients
    print "\nActual Coefficients:\n %s\n" % poly1d(chebyshev8coeffs)

    # plot solution versus exact coefficients
    plot_solution(solution) 
    getch()

# end of file



#!/usr/bin/env python
#Adapted from parallel_desolve.py by mmckerns@caltech.edu

__doc__ = """
# Tests MPI version of Storn and Price's Polynomial 'Fitting' Problem.
# 
# Exact answer: [1,1,1]
  
# Reference:
#
# [1] Storn, R. and Price, K. Differential Evolution - A Simple and Efficient
# Heuristic for Global Optimization over Continuous Spaces. Journal of Global
# Optimization 11: 341-359, 1997.

# To run in parallel:  (must install 'pyina')
mpipython.exe ezmap_desolve_rosen.py
"""

try:
  import pyina.launchers as launchers
  from pyina.launchers import mpirun_launcher
  from pyina.mappers import equalportion_mapper
  from pyina.ez_map import ez_map2 as ez_map
except:
  print __doc__


from mystic.differential_evolution import DifferentialEvolutionSolver2
from mystic.termination import ChangeOverGeneration, VTR
from mystic.strategy import Best1Exp
from mystic.tools import VerboseSow, random_seed

#from raw_rosen import rosen as myCost     # ez_map needs a helper function
from mystic.models import rosen as myCost  # ez_map2 doesn't require help

ND = 3
NP = ND*10
MAX_GENERATIONS = ND*NP
NNODES = ND

TOL = 0.01
CROSS = 0.9
SCALE = 0.8
seed = 100


if __name__=='__main__':
    def print_solution(func):
        print func
        return

    psow = VerboseSow(10)
    ssow = VerboseSow(10)

    random_seed(seed)
    print "first sequential..."
    solver = DifferentialEvolutionSolver2(ND,NP)  #XXX: sequential
    solver.SetRandomInitialPoints(min=[-100.0]*ND, max=[100.0]*ND)
    solver.SetEvaluationLimits(maxiter=MAX_GENERATIONS)
    solver.Solve(myCost, VTR(TOL), strategy=Best1Exp, \
                StepMonitor=ssow, CrossProbability=CROSS, ScalingFactor=SCALE, \
                disp=1)
    print ""
    print_solution( solver.Solution() )

    random_seed(seed)
    print "\n and now parallel..."
    solver2 = DifferentialEvolutionSolver2(ND,NP)  #XXX: parallel
    solver2.SetMapper(ez_map, equalportion_mapper)
    solver2.SetLauncher(mpirun_launcher, NNODES)
    solver2.SetRandomInitialPoints(min=[-100.0]*ND, max=[100.0]*ND)
    solver2.SetEvaluationLimits(maxiter=MAX_GENERATIONS)
    solver2.Solve(myCost, VTR(TOL), strategy=Best1Exp, \
                StepMonitor=psow, CrossProbability=CROSS, ScalingFactor=SCALE, \
                disp=1)
    print ""
    print_solution( solver2.Solution() )

# end of file



#!/usr/bin/env python
# 
# Patrick Hung.

"""
Simple utility functions for SV-classifications
"""

from numpy import zeros, multiply, ndarray, vectorize, array, dot, transpose, diag, sum

def KernelMatrix(X, k=dot):
    n = X.shape[0]
    Q = zeros((n,n))
    for i in range(n):
       for j in range(i, n):
           Q[i,j] = k(X[i,:],X[j,:])
    return Q + transpose(Q) - diag(Q.diagonal())

def WeightVector(alpha, X, y):
    ay = (alpha * y).flatten()
    aXy = transpose(ay * transpose(X))
    return sum(aXy, 0)


def SupportVectors(alpha, y=None, eps = 0):
    import mystic.svmtools
    sv = svmtools.SupportVectors(alpha,eps)
    if y == None:
        return sv
    else:
        class1 = set((y>0).nonzero()[1])
        class2 = set((y<0).nonzero()[1])
        sv1 = class1.intersection(sv)
        sv2 = class2.intersection(sv)
        return list(sv1), list(sv2)

def Bias(alpha, X, y, kernel=dot):
    """Compute classification bias. """
    sv1, sv2 = SupportVectors(alpha, y,eps=1e-6)
    pt1, pt2 = X[sv1[0],:], X[sv2[0],:]
    k1, k2 = kernel(X, pt1), kernel(X,pt2)
    return -0.5 * (sum(alpha*y*k1) + sum(alpha*y*k2))

# end of file



#!/usr/bin/env python

"""
Zimmermann's function

References::
    [1] Storn, R. and Price, K. Differential Evolution - A Simple and Efficient
    Heuristic for Global Optimization over Continuous Spaces. Journal of Global
    Optimization 11: 341-359, 1997.

    [2] Storn, R. and Price, K.
    (Same title as above, but as a technical report.)
    http://www.icsi.berkeley.edu/~storn/deshort1.ps
"""
from abstract_model import AbstractFunction

class Zimmermann(AbstractFunction):
    """Zimmermann function:
a non-continuous function, Equation (24-26) of [2]"""

    def __init__(self):
        AbstractFunction.__init__(self)
        return

    def function(self,coeffs):
        """evaluates the Zimmermann function for a list of coeffs

minimum is f(x)=0.0 at x=(7.0,2.0)"""
        x0, x1 = coeffs #must provide 2 values (x0,y0)
        f8 = 9 - x0 - x1
        c0,c1,c2,c3 = 0,0,0,0
        if x0 < 0: c0 = -100 * x0
        if x1 < 0: c1 = -100 * x1
        xx =  (x0-3.)*(x0-3) + (x1-2.)*(x1-2)
        if xx > 16: c2 = 100 * (xx-16)
        if x0 * x1 > 14: c3 = 100 * (x0*x1-14.)
        return max(f8,c0,c1,c2,c3)

#   def forward(self,pts):
#       """2-D Zimmermann; returns f(xi,yi) for pts=(x,y)"""
#       return AbstractFunction.forward(self,pts)

    pass


# prepared instances
zimmermann = Zimmermann()

# End of file



#!/usr/bin/env python
#
## Differential Evolution Solver Class
## Based on algorithms developed by Dr. Rainer Storn & Kenneth Price
## Original C++ code written by: Lester E. Godwin
##                               PushCorp, Inc.
##                               Dallas, Texas
##                               972-840-0208 x102
##                               godwin@pushcorp.com
## Created: 6/8/98
## Last Modified: 6/8/98         Revision: 1.0
##
## Solver code ported to Python from C++ July 2002
## by: James R. Phillips
##     Birmingham, Alabama USA
##     zunzun@zunzun.com
##
## DE Solver modified and cleaned by Patrick Hung, May 2006.
## additional DE Solver (DESolver2) added by Patrick Hung.
##
## bounds (and minimal interface) added by mmckerns@caltech.edu
## adapted to AbstractSolver interface by mmckerns@caltech.edu
##
## modified for AbstractMapSolver interface by mmckerns@caltech.edu 

"""
Solvers
=======

This module contains a collection of optimization routines based on
Storn and Price's differential evolution algorithm.  The core solver
algorithm was adapted from Phillips's DETest.py.  An alternate solver
is provided that follows the logic in Price, Storn, and Lampen -- in that
both a current generation and a trial generation are maintained, and all
vectors for creating difference vectors and mutations draw from the
current generation... which remains invariant until the end of the
iteration.

A minimal interface that mimics a scipy.optimize interface has also been
implemented, and functionality from the mystic solver API has been added
with reasonable defaults.  DifferentialEvolutionSolver2 is used by
the minimal interface, unless 'invariant_current' is set to False.

Minimal function interface to optimization routines::
    diffev      -- Differential Evolution (DE) solver

The corresponding solvers built on mystic's AbstractSolver are::
    DifferentialEvolutionSolver  -- a DE solver
    DifferentialEvolutionSolver2 -- Storn & Price's DE solver

Mystic solver behavior activated in deffev::
    - EvaluationMonitor = Sow()
    - StepMonitor = Sow()
    - enable_signal_handler()
    - strategy = Best1Exp
    - termination = ChangeOverGeneration(ftol,gtol), if gtol provided
          ''      = VTR(ftol), otherwise

Storn & Price's DE Solver has also been implemented to use the "map"
interface. Mystic enables the user to override the standard python
map function with their own 'map' function, or one of the map functions
provided by the pathos package (see http://dev.danse.us/trac/pathos)
for distributed and high-performance computing.


Usage
=====

Practical advice for how to configure the Differential Evolution
Solver for your own objective function can be found on R. Storn's
web page (http://www.icsi.berkeley.edu/~storn/code.html), and is
reproduced here::

    First try the following classical settings for the solver configuration:
    Choose a crossover strategy (e.g. Rand1Bin), set the number of parents
    NP to 10 times the number of parameters, select ScalingFactor=0.8, and
    CrossProbability=0.9.

    It has been found recently that selecting ScalingFactor from the interval
    [0.5, 1.0] randomly for each generation or for each difference vector,
    a technique called dither, improves convergence behaviour significantly,
    especially for noisy objective functions.

    It has also been found that setting CrossProbability to a low value,
    e.g. CrossProbability=0.2 helps optimizing separable functions since
    it fosters the search along the coordinate axes. On the contrary,
    this choice is not effective if parameter dependence is encountered,
    something which is frequently occuring in real-world optimization
    problems rather than artificial test functions. So for parameter
    dependence the choice of CrossProbability=0.9 is more appropriate.

    Another interesting empirical finding is that rasing NP above, say, 40
    does not substantially improve the convergence, independent of the
    number of parameters. It is worthwhile to experiment with these suggestions.
  
    Make sure that you initialize your parameter vectors by exploiting
    their full numerical range, i.e. if a parameter is allowed to exhibit
    values in the range [-100, 100] it's a good idea to pick the initial
    values from this range instead of unnecessarily restricting diversity.

    Keep in mind that different problems often require different settings
    for NP, ScalingFactor and CrossProbability (see Ref 1, 2). If you
    experience misconvergence, you typically can increase the value for NP,
    but often you only have to adjust ScalingFactor to be a little lower or
    higher than 0.8. If you increase NP and simultaneously lower ScalingFactor
    a little, convergence is more likely to occur but generally takes longer,
    i.e. DE is getting more robust (a convergence speed/robustness tradeoff).

    If you still get misconvergence you might want to instead try a different
    crossover strategy. The most commonly used are Rand1Bin, Rand1Exp,
    Best1Bin, and Best1Exp. The crossover strategy is not so important a
    choice, although K. Price claims that binomial (Bin) is never worse than
    exponential (Exp).

    In case of continued misconvergence, check the choice of objective function.
    There might be a better one to describe your problem. Any knowledge that
    you have about the problem should be worked into the objective function.
    A good objective function can make all the difference.

See `mystic.examples.test_rosenbrock` for an example of using
DifferentialEvolutionSolver. DifferentialEvolutionSolver2 has
the identical interface and usage.

All solvers included in this module provide the standard signal handling.
For more information, see `mystic.mystic.abstract_solver`.


References
==========

[1] Storn, R. and Price, K. Differential Evolution - A Simple and Efficient
Heuristic for Global Optimization over Continuous Spaces. Journal of Global
Optimization 11: 341-359, 1997.

[2] Price, K., Storn, R., and Lampinen, J. - Differential Evolution,
A Practical Approach to Global Optimization. Springer, 1st Edition, 2005

"""
__all__ = ['DifferentialEvolutionSolver','DifferentialEvolutionSolver2',\
           'diffev']

from mystic.tools import Null, wrap_function, unpair
from mystic.tools import wrap_bounds

from abstract_solver import AbstractSolver
from abstract_map_solver import AbstractMapSolver

class DifferentialEvolutionSolver(AbstractSolver):
    """
Differential Evolution optimization.
    """
    
    def __init__(self, dim, NP):
        """
Takes two initial inputs: 
    dim  -- dimensionality of the problem
    NP   -- size of the trial solution population. [requires: NP <= 4]

All important class members are inherited from AbstractSolver.
        """
        #XXX: raise Error if npop <= 4?
        AbstractSolver.__init__(self,dim,npop=NP)
        self.genealogy     = [ [] for j in range(NP)]
        self.scale         = 0.7
        self.probability   = 0.5
        
### XXX: OBSOLETED by wrap_bounds ###
#   def _keepSolutionWithinRangeBoundary(self, base):
#       """scale trialSolution to be between base value and range boundary"""
#       if not self._useStrictRange:
#           return
#       min = self._strictMin
#       max = self._strictMax
#       import random
#       for i in range(self.nDim):
#           if base[i] < min[i] or base[i] > max[i]:
#               self.trialSolution[i] = random.uniform(min[i],max[i])
#           elif self.trialSolution[i] < min[i]:
#               self.trialSolution[i] = random.uniform(min[i],base[i])
#           elif self.trialSolution[i] > max[i]:
#               self.trialSolution[i] = random.uniform(base[i],max[i])
#       return

    def UpdateGenealogyRecords(self, id, newchild):
        """
Override me for more refined behavior. Currently all changes
are logged.
        """
        self.genealogy[id].append(newchild)
        return

    def Solve(self, costfunction, termination, sigint_callback=None,
              EvaluationMonitor=Null, StepMonitor=Null, ExtraArgs=(), **kwds):
        """Minimize a function using differential evolution.

Description:

    Uses a differential evolution algorith to find the minimum of
    a function of one or more variables.

Inputs:

    costfunction -- the Python function or method to be minimized.
    termination -- callable object providing termination conditions.

Additional Inputs:

    sigint_callback -- callback function for signal handler.
    EvaluationMonitor -- a callable object that will be passed x, fval
        whenever the cost function is evaluated.
    StepMonitor -- a callable object that will be passed x, fval
        after the end of a simplex iteration.
    ExtraArgs -- extra arguments for func.

Further Inputs:

    strategy -- the mutation strategy for generating new trial
        solutions [default = Best1Bin]
    CrossProbability -- the probability of cross-parameter mutations
        [default = 0.9]
    ScalingFactor -- multiplier for the impact of mutations on the
        trial solution [default = 0.8]
    callback -- an optional user-supplied function to call after each
        iteration.  It is called as callback(xk), where xk is
        the current parameter vector.  [default = None]
    disp -- non-zero to print convergence messages.

        """
        #allow for inputs that don't conform to AbstractSolver interface
        from mystic.strategy import Best1Bin
        strategy=Best1Bin    #mutation strategy (see mystic.strategy)
        CrossProbability=0.9 #potential for parameter cross-mutation
        ScalingFactor=0.8    #multiplier for mutation impact
        callback=None        #user-supplied function, called after each step
        disp=0               #non-zero to print convergence messages
        if kwds.has_key('strategy'): strategy = kwds['strategy']
        if kwds.has_key('CrossProbability'): \
           CrossProbability = kwds['CrossProbability']
        if kwds.has_key('ScalingFactor'): ScalingFactor = kwds['ScalingFactor']
        if kwds.has_key('callback'): callback = kwds['callback']
        if kwds.has_key('disp'): disp = kwds['disp']
        #-------------------------------------------------------------

        import signal
        self._EARLYEXIT = False

        fcalls, costfunction = wrap_function(costfunction, ExtraArgs, EvaluationMonitor)
        if self._useStrictRange:
            for i in range(self.nPop):
                self.population[i] = self._clipGuessWithinRangeBoundary(self.population[i])
            costfunction = wrap_bounds(costfunction, self._strictMin, self._strictMax)

        #generate signal_handler
        self._generateHandler(sigint_callback) 
        if self._handle_sigint: signal.signal(signal.SIGINT, self.signal_handler)

        self.probability = CrossProbability
        self.scale = ScalingFactor

        #set initial solution and energy by running a single iteration
        self.bestSolution = self.population[0][:]
        self.bestEnergy = self.popEnergy[0]
        for candidate in range(self.nPop):
            # generate trialSolution (within valid range)
            self.trialSolution[:] = self.population[candidate][:]
            trialEnergy = costfunction(self.trialSolution)

            if trialEnergy < self.popEnergy[candidate]:
                # New low for this candidate
                self.popEnergy[candidate] = trialEnergy
                self.population[candidate][:] = self.trialSolution[:]
                self.UpdateGenealogyRecords(candidate, self.trialSolution[:])

                # Check if all-time low
                if trialEnergy < self.bestEnergy:
                    self.bestEnergy = trialEnergy
                    self.bestSolution[:] = self.trialSolution[:]
                            
        self.energy_history.append(self.bestEnergy)
        StepMonitor(self.bestSolution[:], self.bestEnergy)
        self.generations = 0  #XXX: above currently *not* counted as an iteration
        if callback is not None:
            callback(self.bestSolution)
         
        if self._maxiter is None:
            self._maxiter = self.nDim * self.nPop * 10  #XXX: set better defaults?
        if self._maxfun is None:
            self._maxfun = self.nDim * self.nPop * 1000 #XXX: set better defaults?

        #run for generations <= maxiter
        for generation in range(self._maxiter - self.generations):
            if fcalls[0] >= self._maxfun: break
            for candidate in range(self.nPop):
                # generate trialSolution (within valid range)
                strategy(self, candidate)
                trialEnergy = costfunction(self.trialSolution)

                if trialEnergy < self.popEnergy[candidate]:
                    # New low for this candidate
                    self.popEnergy[candidate] = trialEnergy
                    self.population[candidate][:] = self.trialSolution[:]
                    self.UpdateGenealogyRecords(candidate, self.trialSolution[:])

                    # Check if all-time low
                    if trialEnergy < self.bestEnergy:
                        self.bestEnergy = trialEnergy
                        self.bestSolution[:] = self.trialSolution[:]
                            
            self.energy_history.append(self.bestEnergy)
            StepMonitor(self.bestSolution[:], self.bestEnergy)
            self.generations += 1
            if callback is not None:
                callback(self.bestSolution)
            
            if self._EARLYEXIT or termination(self):
                break

        signal.signal(signal.SIGINT,signal.default_int_handler)

        # code below here pushes output to scipy.optimize.fmin interface
        fval = self.bestEnergy
        warnflag = 0

        if fcalls[0] >= self._maxfun:
            warnflag = 1
            if disp:
                print "Warning: Maximum number of function evaluations has "\
                      "been exceeded."
        elif self.generations >= self._maxiter:
            warnflag = 2
            if disp:
                print "Warning: Maximum number of iterations has been exceeded"
        else:
            if disp:
                print "Optimization terminated successfully."
                print "         Current function value: %f" % fval
                print "         Iterations: %d" % self.generations
                print "         Function evaluations: %d" % fcalls[0]

        return 



class DifferentialEvolutionSolver2(AbstractMapSolver):
    """
Differential Evolution optimization, using Storn and Price's algorithm.

Alternate implementation: 
    - utilizes a map-reduce interface, extensible to parallel computing
    - both a current and a next generation are kept, while the current
      generation is invariant during the main DE logic
    """
    def __init__(self, dim, NP):
        """
Takes two initial inputs: 
    dim  -- dimensionality of the problem
    NP   -- size of the trial solution population. [requires: NP <= 4]

All important class members are inherited from AbstractSolver.
        """
        #XXX: raise Error if npop <= 4?
        super(DifferentialEvolutionSolver2, self).__init__(dim, npop=NP)
        self.genealogy     = [ [] for j in range(NP)]
        self.scale         = 0.7
        self.probability   = 0.5
        
    def UpdateGenealogyRecords(self, id, newchild):
        """
Override me for more refined behavior. Currently all changes
are logged.
        """
        self.genealogy[id].append(newchild)
        return

    def Solve(self, costfunction, termination, sigint_callback=None,
              EvaluationMonitor=Null, StepMonitor=Null, ExtraArgs=(), **kwds):
        """Minimize a function using differential evolution.

Description:

    Uses a differential evolution algorith to find the minimum of
    a function of one or more variables. This implementation holds
    the current generation invariant until the end of each iteration.

Inputs:

    costfunction -- the Python function or method to be minimized.
    termination -- callable object providing termination conditions.

Additional Inputs:

    sigint_callback -- callback function for signal handler.
    EvaluationMonitor -- a callable object that will be passed x, fval
        whenever the cost function is evaluated.
    StepMonitor -- a callable object that will be passed x, fval
        after the end of a simplex iteration.
    ExtraArgs -- extra arguments for func.

Further Inputs:

    strategy -- the mutation strategy for generating new trial
        solutions [default = Best1Bin]
    CrossProbability -- the probability of cross-parameter mutations
        [default = 0.9]
    ScalingFactor -- multiplier for the impact of mutations on the
        trial solution [default = 0.8]
    callback -- an optional user-supplied function to call after each
        iteration.  It is called as callback(xk), where xk is
        the current parameter vector.  [default = None]
    constraints -- an optional user-supplied function to call just
        prior to each function evaluation.  It is called as
        xk' = constraints(xk), where xk is the current parameter vector.
        Ideally, this function is used to ensure each function evaulation
        encounters a parameter set that satisfies user-supplied constraints.
    disp -- non-zero to print convergence messages.
        """
        #allow for inputs that don't conform to AbstractSolver interface
        from mystic.strategy import Best1Bin
        strategy=Best1Bin    #mutation strategy (see mystic.strategy)
        CrossProbability=0.9 #potential for parameter cross-mutation
        ScalingFactor=0.8    #multiplier for mutation impact
        callback=None        #user-supplied function, called after each step
        disp=0               #non-zero to print convergence messages
        if kwds.has_key('strategy'): strategy = kwds['strategy']
        if kwds.has_key('CrossProbability'): \
           CrossProbability = kwds['CrossProbability']
        if kwds.has_key('ScalingFactor'): ScalingFactor = kwds['ScalingFactor']
        if kwds.has_key('callback'): callback = kwds['callback']
        if kwds.has_key('disp'): disp = kwds['disp']
        #XXX:[HACK] accept constraints
        constraints = lambda x: x
        if kwds.has_key('constraints'): constraints = kwds['constraints']
        #XXX:[HACK] -end-
        #-------------------------------------------------------------

        import signal
        self._EARLYEXIT = False

       #FIXME: EvaluationMonitor fails for MPI, throws error for 'pp'
        from python_map import python_map
        if self._map != python_map:
            fcalls = [0] #FIXME: temporary patch for removing the following line
        else:
            fcalls, costfunction = wrap_function(costfunction, ExtraArgs, EvaluationMonitor)
        if self._useStrictRange:
            for i in range(self.nPop):
                self.population[i] = self._clipGuessWithinRangeBoundary(self.population[i])
            costfunction = wrap_bounds(costfunction, self._strictMin, self._strictMax)

        #generate signal_handler
        self._generateHandler(sigint_callback) 
        if self._handle_sigint: signal.signal(signal.SIGINT, self.signal_handler)

        self.probability = CrossProbability
        self.scale = ScalingFactor

        #set initial solution and energy by running a single iteration
        self.bestSolution = self.population[0][:]
        self.bestEnergy = self.popEnergy[0]
        trialPop = [[0.0 for i in range(self.nDim)] for j in range(self.nPop)]
        for candidate in range(self.nPop):
            # generate trialSolution (within valid range)
            self.trialSolution[:] = self.population[candidate][:]
            #XXX:[HACK] apply constraints
            self.trialSolution[:] = constraints(self.trialSolution[:])
            #XXX:[HACK] -end-
            trialPop[candidate][:] = self.trialSolution[:]

        mapconfig = dict(nnodes=self._nnodes, launcher=self._launcher, \
                         mapper=self._mapper, queue=self._queue, \
                         timelimit=self._timelimit, \
                         ncpus=self._ncpus, servers=self._servers)
        trialEnergy = self._map(costfunction, trialPop, **mapconfig)

        for candidate in range(self.nPop):
            if trialEnergy[candidate] < self.popEnergy[candidate]:
                # New low for this candidate
                self.popEnergy[candidate] = trialEnergy[candidate]
                self.population[candidate][:] = trialPop[candidate][:]
                self.UpdateGenealogyRecords(candidate, trialPop[candidate][:])

                # Check if all-time low
                if trialEnergy[candidate] < self.bestEnergy:
                    self.bestEnergy = trialEnergy[candidate]
                    self.bestSolution[:] = trialPop[candidate][:]
                        
        self.energy_history.append(self.bestEnergy)
       #FIXME: StepMonitor works for 'pp'?
        StepMonitor(self.bestSolution[:], self.bestEnergy)
        self.generations = 0  #XXX: above currently *not* counted as an iteration
        if callback is not None:
            callback(self.bestSolution)
         
        if self._maxiter is None:
            self._maxiter = self.nDim * self.nPop * 10  #XXX: set better defaults?
        if self._maxfun is None:
            self._maxfun = self.nDim * self.nPop * 1000 #XXX: set better defaults?

        #run for generations <= maxiter
        for generation in range(self._maxiter - self.generations):
            if fcalls[0] >= self._maxfun: break
            for candidate in range(self.nPop):
                # generate trialSolution (within valid range)
                strategy(self, candidate)
                #XXX:[HACK] apply constraints
                self.trialSolution[:] = constraints(self.trialSolution[:])
                #XXX:[HACK] -end-
                trialPop[candidate][:] = self.trialSolution[:]

            trialEnergy = self._map(costfunction, trialPop, **mapconfig)
           ##XXX:[HACK] return trialPop b/c may be altered by constraints
           #trials = map(costfunction, trialPop)
           #trialEnergy = [i for i,j in trials]
           #trialPop = [j for i,j in trials] #FIXME: breaks on inf (out of bounds)
           ##XXX:[HACK] -end-

            for candidate in range(self.nPop):
                if trialEnergy[candidate] < self.popEnergy[candidate]:
                    # New low for this candidate
                    self.popEnergy[candidate] = trialEnergy[candidate]
                    self.population[candidate][:] = trialPop[candidate][:]
                    self.UpdateGenealogyRecords(candidate, trialPop[candidate][:])

                    # Check if all-time low
                    if trialEnergy[candidate] < self.bestEnergy:
                        self.bestEnergy = trialEnergy[candidate]
                        self.bestSolution[:] = trialPop[candidate][:]
                            
            self.energy_history.append(self.bestEnergy)
           #FIXME: StepMonitor works for 'pp'?
            StepMonitor(self.bestSolution[:], self.bestEnergy)
            self.generations += 1
            if callback is not None:
                callback(self.bestSolution)
            
            if self._EARLYEXIT or termination(self):
                break

        signal.signal(signal.SIGINT,signal.default_int_handler)

        # code below here pushes output to scipy.optimize.fmin interface
        fval = self.bestEnergy
        warnflag = 0

        if fcalls[0] >= self._maxfun:
            warnflag = 1
            if disp:
                print "Warning: Maximum number of function evaluations has "\
                      "been exceeded."
        elif self.generations >= self._maxiter:
            warnflag = 2
            if disp:
                print "Warning: Maximum number of iterations has been exceeded"
        else:
            if disp:
                print "Optimization terminated successfully."
                print "         Current function value: %f" % fval
                print "         Iterations: %d" % self.generations
                print "         Function evaluations: %d" % fcalls[0]

        return 


def diffev(func,x0,npop,args=(),bounds=None,ftol=5e-3,gtol=None,
           maxiter=None,maxfun=None,cross=1.0,scale=0.9,
           full_output=0,disp=1,retall=0,callback=None,invariant_current=True):
    """Minimize a function using differential evolution.

Description:

    Uses a differential evolution algorith to find the minimum of
    a function of one or more variables. Mimics a scipy.optimize style
    interface.

Inputs:

    func -- the Python function or method to be minimized.
    x0 -- the initial guess (ndarray), if desired to start from a
        set point; otherwise takes an array of (min,max) bounds,
        for when random initial points are desired
    npop -- size of the trial solution population.

Additional Inputs:

    args -- extra arguments for func.
    bounds -- list - n pairs of bounds (min,max), one pair for each parameter.
    ftol -- number - acceptable relative error in func(xopt) for convergence.
    gtol -- number - maximum number of iterations to run without improvement.
    maxiter -- number - the maximum number of iterations to perform.
    maxfun -- number - the maximum number of function evaluations.
    cross -- number - the probability of cross-parameter mutations
    scale -- number - multiplier for impact of mutations on trial solution.
    full_output -- number - non-zero if fval and warnflag outputs are desired.
    disp -- number - non-zero to print convergence messages.
    retall -- number - non-zero to return list of solutions at each iteration.
    callback -- an optional user-supplied function to call after each
        iteration.  It is called as callback(xk), where xk is the
        current parameter vector.
    invariant_current -- set to True to call DifferentialEvolutionSolver2,
        otherwise call DifferentialEvolutionSolver

Returns: (xopt, {fopt, iter, funcalls, warnflag}, {allvecs})

    xopt -- ndarray - minimizer of function
    fopt -- number - value of function at minimum: fopt = func(xopt)
    iter -- number - number of iterations
    funcalls -- number - number of function calls
    warnflag -- number - Integer warning flag:
        1 : 'Maximum number of function evaluations.'
        2 : 'Maximum number of iterations.'
    allvecs -- list - a list of solutions at each iteration

    """

    from mystic.tools import Sow
    stepmon = Sow()
    evalmon = Sow()
   #from mystic.strategy import Best1Exp #, Best1Bin, Rand1Exp
   #strategy = Best1Exp
    if gtol: #if number of generations provided, use ChangeOverGeneration 
        from mystic.termination import ChangeOverGeneration
        termination = ChangeOverGeneration(ftol,gtol)
    else:
        from mystic.termination import VTR
        termination = VTR(ftol)

    ND = len(x0)
    if invariant_current: #use Solver2, not Solver1
        solver = DifferentialEvolutionSolver2(ND,npop)
    else:
        solver = DifferentialEvolutionSolver(ND,npop)
    solver.SetEvaluationLimits(maxiter,maxfun)
    if bounds:
        minb,maxb = unpair(bounds)
        solver.SetStrictRanges(minb,maxb)

    try: #x0 passed as 1D array of (min,max) pairs
        minb,maxb = unpair(x0)
        solver.SetRandomInitialPoints(minb,maxb)
    except: #x0 passed as 1D array of initial parameter values
        solver.SetInitialPoints(x0)

    solver.enable_signal_handler()
    #TODO: allow sigint_callbacks for all minimal interfaces ?
    solver.Solve(func,termination=termination,\
                #strategy=strategy,sigint_callback=other_callback,\
                 CrossProbability=cross,ScalingFactor=scale,\
                 EvaluationMonitor=evalmon,StepMonitor=stepmon,\
                 ExtraArgs=args,callback=callback)
    solution = solver.Solution()

    # code below here pushes output to scipy.optimize.fmin interface
   #x = list(solver.bestSolution)
    x = solver.bestSolution
    fval = solver.bestEnergy
    warnflag = 0
    fcalls = len(evalmon.x)
    iterations = len(stepmon.x)
    allvecs = stepmon.x

    if fcalls >= solver._maxfun:
        warnflag = 1
        if disp:
            print "Warning: Maximum number of function evaluations has "\
                  "been exceeded."
    elif iterations >= solver._maxiter:
        warnflag = 2
        if disp:
            print "Warning: Maximum number of iterations has been exceeded"
    else:
        if disp:
            print "Optimization terminated successfully."
            print "         Current function value: %f" % fval
            print "         Iterations: %d" % iterations
            print "         Function evaluations: %d" % fcalls

    if full_output:
        retlist = x, fval, iterations, fcalls, warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = x
        if retall:
            retlist = (x, allvecs)

    return retlist


if __name__=='__main__':
    help(__name__)

# end of file
